const path = require("path")
const util = require("../util")
const webpack = require("webpack")
const autoprefixer = require('autoprefixer')
const ExtractTextPlugin = require('extract-text-webpack-plugin')
const OptimizeCSSPlugin = require('optimize-css-assets-webpack-plugin')
const ChunkRenamePlugin = require("./chunk-rename-webpack-plugin")
const HtmlWebpackPlugin = require('html-webpack-plugin')
const devModuleRules = require("./webpack.config.dev").module.rules
const { userConfig, webpackConfig, templates } = require("./webpack.config.base")
const context = webpackConfig.context
let publicPath = userConfig.publicPath
// 默认自己公司的配置
if (!publicPath) {
  let publicDomain = '//static.wuage.com/'
  publicPath = publicDomain + path.basename(context).replace('style-', '') + '/'
}

// 异步模块打包 @todo 临时解决方案
const basePath = templates[0].basePath
let config = module.exports = Object.assign({}, webpackConfig, {
  output: Object.assign({}, webpackConfig.output, {
    publicPath: publicPath,
    // 输出的文件名 hash统一生成,chunkhash变化生成
    filename: userConfig.chunkhash ? '[name].[chunkhash:' + userConfig.chunkhash + '].js' : '[name].js',
    chunkFilename: basePath + '/[id].[chunkhash:8].js'
  }),
  module: {
    rules: userConfig.extractCSS ? webpackConfig.module.rules.concat(
      // vue
      {
        test: /\.vue$/,
        loader: 'vue-loader',
        options: {
          loaders: util.extractVueStlye(),
          preserveWhitespace: false
        }
      },
      // style
      util.stlyeRules(userConfig["devtool"], true)
    ) : devModuleRules
  },
  plugins: [
    // 配置环境
    new webpack.DefinePlugin({
      'process.env': {
        NODE_ENV: '"production"'
      }
    }),

    // 对js进行压缩混淆
    new webpack.optimize.UglifyJsPlugin({
      compress: {
        warnings: false,
        drop_debugger: true,
        drop_console: true
      },
      sourceMap: true
    }),

    // 对css压缩
    new webpack.LoaderOptionsPlugin({
      minimize: true
    }),

    // 当模块内容没有修改时，保持module.id不变
    new webpack.HashedModuleIdsPlugin(),

    // new ManifestPlugin({
    //   fileName: 'asset-manifest.json'
    // }),
    // 这个和全路径目录冲突异步块重命名， basePath
    // new ChunkRenamePlugin(templates)
  ],
  // 上线设置source-map会影响加载速度,但便于线上bug查找
  devtool: userConfig["devtool"],

  //编译期间一旦出错就停止编译
  bail: true
})

// webpack3.0  模块作用域提升
if (webpack.optimize.ModuleConcatenationPlugin) {
  config.plugins.push(new webpack.optimize.ModuleConcatenationPlugin())
}

// 提取文件的公共部分
if (userConfig && userConfig.vendor) {
  const filename = userConfig.chunkhash ? 'common.[chunkhash:' + userConfig.chunkhash + '].js' : 'common.js'
  templates.forEach((tpl, index) => {
    const common = tpl.basePath + '/common'

    // 在node_modules下的vue组件不提取了，提取了组件中的css就无法提取到公共地方
    // 或者按照extractCSS配置来决定@todo
    const minChunks = ({ resource }, count) => (
      resource &&
      ~resource.indexOf('node_modules') &&
      resource.match(/\.js$/)
    ) || (count > 1)

    config.plugins.push(
      new webpack.optimize.CommonsChunkPlugin({
        name: common,
        filename: tpl.basePath + '/' + filename,
        chunks: [tpl.chunk],
        minChunks,
      })
    )

    // 提取异步共享的模块, 未解决
    config.plugins.push(
      new webpack.optimize.CommonsChunkPlugin({
        name: tpl.chunk,
        async: basePath + '/async',
        // chunks: [tpl.chunk],
        minChunks
      })
    )

    const templateName = path.join(tpl.basePath, 'index.html')
    config.plugins.push(new HtmlWebpackPlugin({
      filename: templateName,
      template: tpl.template,
      chunks: [common, tpl.chunk],
      inject: true
    }))
  })
} else {
  // 模板文件
  templates.forEach((tpl) => {
    const filename = path.join(tpl.basePath, 'index.html')
    config.plugins.push(new HtmlWebpackPlugin({
      filename,
      template: tpl.template,
      chunks: [tpl.chunk],
      inject: true
    }))
  })
}

if (userConfig.extractCSS) {
  config.plugins.push(
    // 提取css,生成对应对文件
    new ExtractTextPlugin({
      filename: userConfig.chunkhash ? '[name].[chunkhash:' + userConfig.chunkhash + '].css' : '[name].css',
      allChunks: true
    }),
    // 提取的css可能存在重复的样式，去掉重复的
    new OptimizeCSSPlugin({
      cssProcessorOptions: {
        safe: true
      }
    })
  )
}

if (userConfig.bundleAnalyzerReport) {
  var BundleAnalyzerPlugin = require('webpack-bundle-analyzer').BundleAnalyzerPlugin
  config.plugins.push(new BundleAnalyzerPlugin())
}