const webpackConfig = require("./webpack.config.dev")
const HtmlWebpackPlugin = require('html-webpack-plugin')
const webpack = require('webpack')
const util = require('../util')
const $path = require('path')
const FriendlyErrorsPlugin = require('friendly-errors-webpack-plugin')
const chalk = require('chalk')
const { templates } = require('./webpack.config.base')
const context = webpackConfig.context
const webpackClientPath = $path.join(__dirname, 'webpack.client.js')

// 对每一个入口文件添加 hot-reload 依赖
Object.keys(webpackConfig.entry).forEach(function (name) {
  webpackConfig.entry[name] = [webpackClientPath].concat(webpackConfig.entry[name])
})

// html插件
const htmlPlugin = []

templates.forEach((tpl, index) => {
  if (templates.length === 1) {
    htmlPlugin.push(new HtmlWebpackPlugin({
      template: tpl.template
    }))
  } else {
    if (!util.isFile($path.resolve(context, tpl.template))) {
      console.log(chalk.bold.red('\n[wag][start]' + tpl.template + '模板路径没找到\n'));
      process.exit(1)
    }
    console.log(`\n访问文件的地址：/${tpl.filename}\n`)
    htmlPlugin.push(new HtmlWebpackPlugin({
      filename: tpl.filename,
      template: tpl.template,
      chunks: [tpl.chunk]
    }))

    // 添加一个入口文件
    if (index === 0) {
      htmlPlugin.push(new HtmlWebpackPlugin({
        pages: templates,
        template: $path.join(__dirname, 'index.hbs'),
        inject: false
      }))
    }
  }
})

module.exports = Object.assign({}, webpackConfig, {
  output: Object.assign({}, webpackConfig.output, {
    publicPath: '/',
  }),
  plugins: [
    new webpack.DefinePlugin({
      'process.env': {
        NODE_ENV: '"development"'
      }
    }),
    new webpack.HotModuleReplacementPlugin(),
    new webpack.NoEmitOnErrorsPlugin(),
    new webpack.NamedModulesPlugin(),
    new FriendlyErrorsPlugin()
  ].concat(htmlPlugin)
})
